@@ -7,16 +7,17 @@
 #include <sstream>
 #include <fstream>
 
-#include <core/api/NstApiMachine.hpp>
-#include <core/api/NstApiEmulator.hpp>
-#include <core/api/NstApiVideo.hpp>
-#include <core/api/NstApiSound.hpp>
-#include <core/api/NstApiInput.hpp>
-#include <core/api/NstApiCartridge.hpp>
-#include <core/api/NstApiUser.hpp>
-#include <core/api/NstApiFds.hpp>
+#include "../source/core/api/NstApiMachine.hpp"
+#include "../source/core/api/NstApiEmulator.hpp"
+#include "../source/core/api/NstApiVideo.hpp"
+#include "../source/core/api/NstApiCheats.hpp"
+#include "../source/core/api/NstApiSound.hpp"
+#include "../source/core/api/NstApiInput.hpp"
+#include "../source/core/api/NstApiCartridge.hpp"
+#include "../source/core/api/NstApiUser.hpp"
+#include "../source/core/api/NstApiFds.hpp"
 
-#define NST_VERSION "1.47-WIP"
+#define NST_VERSION "1.48-WIP"
 
 #ifdef _WIN32
 #define snprintf _snprintf
@@ -32,7 +33,14 @@ static retro_environment_t environ_cb;
 static retro_input_poll_t input_poll_cb;
 static retro_input_state_t input_state_cb;
 
+#ifdef _3DS
+extern "C" void* linearMemAlign(size_t size, size_t alignment);
+extern "C" void linearFree(void* mem);
+static uint32_t* video_buffer = NULL;
+#else
 static uint32_t video_buffer[Api::Video::Output::NTSC_WIDTH * Api::Video::Output::HEIGHT];
+#endif
+
 static int16_t audio_buffer[(44100 / 50)];
 static int16_t audio_stereo_buffer[2 * (44100 / 50)];
 static Api::Emulator emulator;
@@ -40,8 +48,11 @@ static Api::Machine *machine;
 static Api::Fds *fds;
 static char g_basename[256];
 static char g_rom_dir[256];
-static bool use_overscan;
+static char *g_save_dir;
 static unsigned blargg_ntsc;
+static bool fds_auto_insert;
+static bool overscan_v;
+static bool overscan_h;
 
 int16_t video_width = Api::Video::Output::WIDTH;
 size_t pitch;
@@ -49,10 +60,12 @@ size_t pitch;
 static Api::Video::Output *video;
 static Api::Sound::Output *audio;
 static Api::Input::Controllers *input;
+static Api::Machine::FavoredSystem favsystem;
 
 static void *sram;
 static unsigned long sram_size;
 static bool is_pal;
+static bool dbpresent;
 
 int crossx = 0;
 int crossy = 0;
@@ -106,7 +119,7 @@ static void NST_CALLBACK file_io_callback(void*, Api::User::File &file)
       case Api::User::File::LOAD_FDS:
          {
             char base[256];
-            snprintf(base, sizeof(base), "%s%c%s.sav", g_rom_dir, slash, g_basename);
+            snprintf(base, sizeof(base), "%s%c%s.sav", g_save_dir, slash, g_basename);
             if (log_cb)
                log_cb(RETRO_LOG_INFO, "Want to load FDS sav from: %s\n", base);
             std::ifstream in_tmp(base,std::ifstream::in|std::ifstream::binary);
@@ -120,7 +133,7 @@ static void NST_CALLBACK file_io_callback(void*, Api::User::File &file)
       case Api::User::File::SAVE_FDS:
          {
             char base[256];
-            snprintf(base, sizeof(base), "%s%c%s.sav", g_rom_dir, slash, g_basename);
+            snprintf(base, sizeof(base), "%s%c%s.sav", g_save_dir, slash, g_basename);
             if (log_cb)
                log_cb(RETRO_LOG_INFO, "Want to save FDS sav to: %s\n", base);
             std::ofstream out_tmp(base,std::ifstream::out|std::ifstream::binary);
@@ -143,6 +156,9 @@ static void check_system_specs(void)
 void retro_init(void)
 {
    struct retro_log_callback log;
+#ifdef _3DS
+   video_buffer = (uint32_t*)linearMemAlign(Api::Video::Output::NTSC_WIDTH * Api::Video::Output::HEIGHT * sizeof(uint32_t), 0x80);
+#endif
 
    machine = new Api::Machine(emulator);
    input = new Api::Input::Controllers;
@@ -174,6 +190,11 @@ void retro_deinit(void)
    video   = 0;
    audio   = 0;
    input   = 0;
+
+#ifdef _3DS
+   linearFree(video_buffer);
+   video_buffer = NULL;
+#endif
 }
 
 unsigned retro_api_version(void)
@@ -201,8 +222,8 @@ void retro_get_system_av_info(struct retro_system_av_info *info)
 
    // It's better if the size is based on NTSC_WIDTH if the filter is on
    const retro_game_geometry geom = {
-      Api::Video::Output::WIDTH,
-      Api::Video::Output::HEIGHT - (use_overscan ? 0 : 16),
+      Api::Video::Output::WIDTH - (overscan_h ? 16 : 0),
+      Api::Video::Output::HEIGHT - (overscan_v ? 16 : 0),
       Api::Video::Output::NTSC_WIDTH,
       Api::Video::Output::HEIGHT,
       4.0 / 3.0,
@@ -216,9 +237,14 @@ void retro_set_environment(retro_environment_t cb)
    environ_cb = cb;
 
    static const struct retro_variable vars[] = {
-      { "blargg_ntsc_filter", "Blargg NTSC filter; disabled|composite|svideo|rgb" },
-      { "palette", "Palette; canonical|consumer|alternative|rgb" },
-      { "nospritelimit", "Remove 8-sprites-per-scanline hardware limit; disabled|enabled" },
+      { "nestopia_blargg_ntsc_filter", "Blargg NTSC filter; disabled|composite|svideo|rgb" },
+      { "nestopia_palette", "Palette; consumer|canonical|alternative|rgb|raw" },
+      { "nestopia_nospritelimit", "Remove 8-sprites-per-scanline hardware limit; disabled|enabled" },
+      { "nestopia_fds_auto_insert", "Automatically insert first FDS disk on reset; enabled|disabled" },
+      { "nestopia_overscan_v", "Mask Overscan (Vertical); enabled|disabled" },
+      { "nestopia_overscan_h", "Mask Overscan (Horizontal); disabled|enabled" },
+      { "nestopia_genie_distortion", "Game Genie Sound Distortion; disabled|enabled" },
+      { "nestopia_favored_system", "Favored System; auto|ntsc|pal|famicom|dendy" },
       { NULL, NULL },
    };
 
@@ -257,7 +283,8 @@ void retro_reset(void)
    if (machine->Is(Nes::Api::Machine::DISK))
    {
       fds->EjectDisk();
-      fds->InsertDisk(0, 0);
+      if (fds_auto_insert)
+         fds->InsertDisk(0, 0);
    }
 }
 
@@ -283,12 +310,14 @@ static void update_input()
    input_poll_cb();
    input->pad[0].buttons = 0;
    input->pad[1].buttons = 0;
+   input->pad[2].buttons = 0;
+   input->pad[3].buttons = 0;
    input->zapper.fire = 0;
    input->vsSystem.insertCoin = 0;
    
    if (Api::Input(emulator).GetConnectedController(1) == 5) {
-      static int zapx = 0; 
-      static int zapy = use_overscan ? 0 : 8;
+      static int zapx = overscan_h ? 8 : 0; 
+      static int zapy = overscan_v ? 8 : 0;
       zapx += input_state_cb(1, RETRO_DEVICE_LIGHTGUN, 0, RETRO_DEVICE_ID_LIGHTGUN_X);
       zapy += input_state_cb(1, RETRO_DEVICE_LIGHTGUN, 0, RETRO_DEVICE_ID_LIGHTGUN_Y);
    
@@ -312,7 +341,7 @@ static void update_input()
       }
    }
    
-   for (unsigned p = 0; p < 2; p++)
+   for (unsigned p = 0; p < 4; p++)
       for (unsigned bind = 0; bind < sizeof(bindmap) / sizeof(bindmap[0]); bind++)
          input->pad[p].buttons |= input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, bindmap[bind].retro) ? bindmap[bind].nes : 0;
          
@@ -324,16 +353,26 @@ static void update_input()
       
    if (machine->Is(Nes::Api::Machine::DISK))
    {
-      if (input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L) &&
-            fds->CanChangeDiskSide())
-         fds->ChangeSide();
+      bool curL = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L);
+      static bool prevL = false;
+      if (curL && !prevL)
+      {
+         if (!fds->IsAnyDiskInserted())
+            fds->InsertDisk(0, 0);
+         else if (fds->CanChangeDiskSide())
+            fds->ChangeSide();
+      }
+      prevL = curL;
       
-      if (input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R) && (fds->GetNumDisks() > 1))
+      bool curR = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R);
+      static bool prevR = false;
+      if (curR && !prevR && (fds->GetNumDisks() > 1))
       {
          int currdisk = fds->GetCurrentDisk();
          fds->EjectDisk();
          fds->InsertDisk(!currdisk, 0);
       }
+      prevR = curR;
    }
 }
 
@@ -342,13 +381,76 @@ static void check_variables(void)
    static bool last_ntsc_val_same;
    struct retro_variable var = {0};
 
-   retro_reset();
+   Api::Sound sound(emulator);
    Api::Video video(emulator);
    Api::Video::RenderState renderState;
-   Api::Machine machine( emulator );
+   Api::Machine machine(emulator);
    Api::Video::RenderState::Filter filter;
-   
-   var.key = "nospritelimit";
+
+   var.key = "nestopia_favored_system";
+   is_pal = false;
+
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
+   {
+      if (strcmp(var.value, "auto") == 0)
+      {
+         if (dbpresent)
+         {
+            machine.SetMode(machine.GetDesiredMode());
+            if (machine.GetMode() == Api::Machine::PAL)
+            {
+               is_pal = true;
+               favsystem = Api::Machine::FAVORED_NES_PAL;
+               machine.SetMode(Api::Machine::PAL);
+            }
+            else
+            {
+               favsystem = Api::Machine::FAVORED_NES_NTSC;
+               machine.SetMode(Api::Machine::NTSC);
+            }
+         }
+      }
+      else if (strcmp(var.value, "ntsc") == 0)
+      {
+         favsystem = Api::Machine::FAVORED_NES_NTSC;
+         machine.SetMode(Api::Machine::NTSC);
+      }
+      else if (strcmp(var.value, "pal") == 0)
+      {
+         favsystem = Api::Machine::FAVORED_NES_PAL;
+         machine.SetMode(Api::Machine::PAL);
+         is_pal = true;
+      }
+      else if (strcmp(var.value, "famicom") == 0)
+      {
+         favsystem = Api::Machine::FAVORED_FAMICOM;
+         machine.SetMode(Api::Machine::NTSC);
+      }
+      else if (strcmp(var.value, "dendy") == 0)
+      {
+         favsystem = Api::Machine::FAVORED_DENDY;
+         machine.SetMode(Api::Machine::PAL);
+      }
+      else
+      {
+         favsystem = Api::Machine::FAVORED_NES_NTSC;
+         machine.SetMode(Api::Machine::NTSC);
+      }
+   }
+   if (audio) delete audio;
+   audio = new Api::Sound::Output(audio_buffer, is_pal ? 44100 / 50 : 44100 / 60);
+
+   var.key = "nestopia_genie_distortion";
+
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
+   {
+      if (strcmp(var.value, "disabled") == 0)
+         sound.SetGenie(0);
+      else if (strcmp(var.value, "enabled") == 0)
+         sound.SetGenie(1);
+   }
+
+   var.key = "nestopia_nospritelimit";
 
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
    {
@@ -358,8 +460,13 @@ static void check_variables(void)
          video.EnableUnlimSprites(true);
    }
    
-   var.key = "blargg_ntsc_filter";
+   var.key = "nestopia_fds_auto_insert";
+
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
+      fds_auto_insert = (strcmp(var.value, "enabled") == 0);
    
+   var.key = "nestopia_blargg_ntsc_filter";
+
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
    {
       if (strcmp(var.value, "disabled") == 0)
@@ -371,7 +478,7 @@ static void check_variables(void)
       else if (strcmp(var.value, "rgb") == 0)
          blargg_ntsc = 4;
    }
-   
+
    switch(blargg_ntsc)
    {
       case 0:
@@ -411,18 +518,18 @@ static void check_variables(void)
          break;
    }
    
-   var.key = "palette";
+   var.key = "nestopia_palette";
    
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
    {
-      if (strcmp(var.value, "canonical") == 0) {
-         video.GetPalette().SetMode(Api::Video::Palette::MODE_YUV);
-         video.SetDecoder(Api::Video::DECODER_CANONICAL);
-      }
-      else if (strcmp(var.value, "consumer") == 0) {
+      if (strcmp(var.value, "consumer") == 0) {
          video.GetPalette().SetMode(Api::Video::Palette::MODE_YUV);
          video.SetDecoder(Api::Video::DECODER_CONSUMER);
       }
+      else if (strcmp(var.value, "canonical") == 0) {
+         video.GetPalette().SetMode(Api::Video::Palette::MODE_YUV);
+         video.SetDecoder(Api::Video::DECODER_CANONICAL);
+      }
       else if (strcmp(var.value, "alternative") == 0) {
          video.GetPalette().SetMode(Api::Video::Palette::MODE_YUV);
          video.SetDecoder(Api::Video::DECODER_ALTERNATIVE);
@@ -430,8 +537,37 @@ static void check_variables(void)
       else if (strcmp(var.value, "rgb") == 0) {
          video.GetPalette().SetMode(Api::Video::Palette::MODE_RGB);
       }
+      else if (strcmp(var.value, "raw") == 0) {
+         /* outputs raw chroma/level/emphasis in the R/G/B channels
+          * that can be decoded by the frontend (using shaders for example)
+          * the following formulas can be used to extract the
+          * values back from a normalized R/G/B triplet
+          * chroma   = floor((R * 15.0) + 0.5)
+          * level    = floor((G *  3.0) + 0.5)
+          * emphasis = floor((B *  7.0) + 0.5) */
+         unsigned char raw_palette[512][3];
+         int i;
+         for (i = 0; i < 512; i++)
+         {
+            raw_palette[i][0] = (((i >> 0) & 0xF) * 255) / 15;
+            raw_palette[i][1] = (((i >> 4) & 0x3) * 255) / 3;
+            raw_palette[i][2] = (((i >> 6) & 0x7) * 255) / 7;
+         }
+         video.GetPalette().SetMode(Api::Video::Palette::MODE_CUSTOM);
+         video.GetPalette().SetCustom(raw_palette, Api::Video::Palette::EXT_PALETTE);
+      }
    }
    
+   var.key = "nestopia_overscan_v";
+
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
+      overscan_v = (strcmp(var.value, "enabled") == 0);
+   
+   var.key = "nestopia_overscan_h";
+
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
+      overscan_h = (strcmp(var.value, "enabled") == 0);
+   
    pitch = video_width * 4;
    
    renderState.filter = filter;
@@ -443,6 +579,8 @@ static void check_variables(void)
    renderState.bits.mask.b = 0x000000ff;
    if (NES_FAILED(video.SetRenderState( renderState )) && log_cb)
       log_cb(RETRO_LOG_INFO, "Nestopia core rejected render state\n");;
+
+    retro_reset();
 }
 
 void retro_run(void)
@@ -467,11 +605,10 @@ void retro_run(void)
       video = new Api::Video::Output(video_buffer, video_width * sizeof(uint32_t));
    }
    
-   bool overscan = blargg_ntsc || use_overscan;
-   
-   video_cb(video_buffer + (overscan ? 0 : (256 * 8)),
-         video_width,
-         Api::Video::Output::HEIGHT - (overscan ? 0 : 16),
+   // Absolute mess of inline if statements...
+   video_cb(video_buffer + (overscan_v ? ((overscan_h ? 8 : 0) + (blargg_ntsc ? Api::Video::Output::NTSC_WIDTH : Api::Video::Output::WIDTH) * 8) : (overscan_h ? 8 : 0) + 0),
+         video_width - (overscan_h ? 16 : 0),
+         Api::Video::Output::HEIGHT - (overscan_v ? 16 : 0),
          pitch);
 }
 
@@ -522,12 +659,67 @@ bool retro_load_game(const struct retro_game_info *info)
    slash = '/';
 #endif
 
+   struct retro_input_descriptor desc[] = {
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_LEFT,  "D-Pad Left" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_UP,    "D-Pad Up" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_DOWN,  "D-Pad Down" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_RIGHT, "D-Pad Right" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_B,     "B" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_A,     "A" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_X,     "(VSSystem) Coin 1" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_Y,     "(VSSystem) Coin 2" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L,     "(FDS) Disk Side Change" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R,     "(FDS) Eject Disk" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_SELECT,   "Select" },
+      { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_START,    "Start" },
+
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_LEFT,  "D-Pad Left" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_UP,    "D-Pad Up" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_DOWN,  "D-Pad Down" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_RIGHT, "D-Pad Right" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_B,     "B" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_A,     "A" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_X,     "(VSSystem) Coin 1" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_Y,     "(VSSystem) Coin 2" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L,     "(FDS) Disk Side Change" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R,     "(FDS) Eject Disk" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_SELECT,   "Select" },
+      { 1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_START,    "Start" },
+
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_LEFT,  "D-Pad Left" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_UP,    "D-Pad Up" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_DOWN,  "D-Pad Down" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_RIGHT, "D-Pad Right" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_B,     "B" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_A,     "A" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_X,     "(VSSystem) Coin 1" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_Y,     "(VSSystem) Coin 2" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L,     "(FDS) Disk Side Change" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R,     "(FDS) Eject Disk" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_SELECT,   "Select" },
+      { 2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_START,    "Start" },
+
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_LEFT,  "D-Pad Left" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_UP,    "D-Pad Up" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_DOWN,  "D-Pad Down" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_RIGHT, "D-Pad Right" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_B,     "B" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_A,     "A" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_X,     "(VSSystem) Coin 1" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_Y,     "(VSSystem) Coin 2" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L,     "(FDS) Disk Side Change" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R,     "(FDS) Eject Disk" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_SELECT,   "Select" },
+      { 3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_START,    "Start" },
+
+      { 0 },
+   };
+
+   environ_cb(RETRO_ENVIRONMENT_SET_INPUT_DESCRIPTORS, desc);
+
    if (!environ_cb(RETRO_ENVIRONMENT_GET_SYSTEM_DIRECTORY, &dir) || !dir)
       return false;
 
-   if (!environ_cb(RETRO_ENVIRONMENT_GET_OVERSCAN, &use_overscan))
-      use_overscan = true;
-
    snprintf(db_path, sizeof(db_path), "%s%cNstDatabase.xml", dir, slash);
 
    if (log_cb)
@@ -540,12 +732,14 @@ bool retro_load_game(const struct retro_game_info *info)
    {
       database.Load(*db_file);
       database.Enable(true);
+      dbpresent = true;
    }
    else
    {
       if (log_cb)
          log_cb(RETRO_LOG_WARN, "NstDatabase.xml required to detect region and some mappers.\n");
       delete db_file;
+      dbpresent = false;
    }
    
    if (info->path != NULL)
@@ -592,30 +786,18 @@ bool retro_load_game(const struct retro_game_info *info)
       else
          return false;
    }
-
-   Api::Machine::FavoredSystem system = Api::Machine::FAVORED_NES_NTSC;
-   is_pal = false;
-
-   if (machine->Load(ss, system))
-      return false;
-
-   // If you have no Database, no region detection for you.
-   if (database.IsLoaded())
+   
+   if (!environ_cb(RETRO_ENVIRONMENT_GET_SAVE_DIRECTORY, &g_save_dir))
    {
-	   machine->SetMode(machine->GetDesiredMode());
-	   if (machine->GetMode() == Api::Machine::PAL)
-		  is_pal = true;
+      if (log_cb)
+         log_cb(RETRO_LOG_ERROR, "Could not find save directory.\n");
    }
-   else
-      machine->SetMode(Api::Machine::NTSC);
 
-   if (machine->Is(Nes::Api::Machine::DISK))
-      fds->InsertDisk(0, 0);
+   is_pal = false;
+   check_variables();
 
-   if (log_cb)
-      log_cb(RETRO_LOG_INFO, "[Nestopia]: Machine is %s.\n", is_pal ? "PAL" : "NTSC");
-
-   audio = new Api::Sound::Output(audio_buffer, is_pal ? 44100 / 50 : 44100 / 60);
+   if (machine->Load(ss, favsystem))
+      return false;
 
    Api::Video ivideo(emulator);
    ivideo.SetSharpness(Api::Video::DEFAULT_SHARPNESS_RGB);
@@ -639,17 +821,33 @@ bool retro_load_game(const struct retro_game_info *info)
    isound.SetSampleRate(44100);
    isound.SetSpeaker(Api::Sound::SPEAKER_MONO);
 
-   //Api::Input(emulator).ConnectController(0, Api::Input::PAD1);
-   //Api::Input(emulator).ConnectController(1, Api::Input::PAD2);
-   Api::Input(emulator).AutoSelectController(0);
-   Api::Input(emulator).AutoSelectController(1);
-   //Api::Input(emulator).ConnectController(1, Api::Input::ZAPPER);
+   if (dbpresent)
+   {
+      Api::Input(emulator).AutoSelectController(0);
+      Api::Input(emulator).AutoSelectController(1);
+      Api::Input(emulator).AutoSelectController(2);
+      Api::Input(emulator).AutoSelectController(3);
+   }
+   else
+   {
+      Api::Input(emulator).ConnectController(0, Api::Input::PAD1);
+      Api::Input(emulator).ConnectController(1, Api::Input::PAD2);
+      //Api::Input(emulator).ConnectController(1, Api::Input::ZAPPER);
+      Api::Input(emulator).ConnectController(2, Api::Input::PAD3);
+      Api::Input(emulator).ConnectController(3, Api::Input::PAD4);
+   }
 
    machine->Power(true);
 
    check_variables();
+
+   if (fds_auto_insert && machine->Is(Nes::Api::Machine::DISK))
+      fds->InsertDisk(0, 0);
    
    video = new Api::Video::Output(video_buffer, video_width * sizeof(uint32_t));
+   
+   if (log_cb)
+      log_cb(RETRO_LOG_INFO, "[Nestopia]: Machine is %s.\n", is_pal ? "PAL" : "NTSC");
 
    return true;
 }
@@ -717,7 +915,18 @@ size_t retro_get_memory_size(unsigned id)
 }
 
 void retro_cheat_reset(void)
-{}
+{
+   Nes::Api::Cheats cheater(emulator);
+   cheater.ClearCodes();
+}
 
-void retro_cheat_set(unsigned, bool, const char *)
-{}
+void retro_cheat_set(unsigned index, bool enabled, const char *code)
+{
+   Nes::Api::Cheats cheater(emulator);
+   Nes::Api::Cheats::Code ggCode;
+
+   if (Nes::Api::Cheats::GameGenieDecode(code, ggCode) == RESULT_OK)
+      cheater.SetCode(ggCode);
+   if (Nes::Api::Cheats::ProActionRockyDecode(code, ggCode) == RESULT_OK)
+      cheater.SetCode(ggCode);
+}
