@@ -230,6 +230,8 @@ namespace Nes
 						CUSTOM_WH                  = MakeId<    1,  128,   64,  0,  0, CRM_0,  NMT_H,  0 >::ID,
 						CUSTOM_X79B                = MakeId<    3,   32,   32,  8,  0, CRM_0,  NMT_X,  0 >::ID,
 						CUSTOM_ZZ                  = MakeId<   37,  256,  256,  0,  0, CRM_0,  NMT_V,  0 >::ID,
+						// Acclaim
+						ACCLAIM_MCACC              = MakeId<    4,  512,  256,  0,  0, CRM_0,  NMT_X,  1 >::ID,
 						// Active Enterprises
 						AE_STD                     = MakeId<  228, 2048,  512,  0,  0, CRM_0,  NMT_V,  0 >::ID,
 						// AGCI
@@ -443,8 +445,9 @@ namespace Nes
 						NAMCOT_163_1               = MakeId<   19,  512,  256,  8,  0, CRM_0,  NMT_X,  0 >::ID,
 						NAMCOT_163_S_0             = MakeId<   19,  512,  256,  0,  0, CRM_0,  NMT_X,  1 >::ID,
 						NAMCOT_163_S_1             = MakeId<   19,  512,  256,  8,  0, CRM_0,  NMT_X,  1 >::ID,
-						NAMCOT_175                 = MakeId<  210,  512,  128,  8,  0, CRM_0,  NMT_X,  0 >::ID,
-						NAMCOT_340                 = MakeId<  210,  256,  256,  0,  0, CRM_0,  NMT_X,  0 >::ID,
+						NAMCOT_175                 = MakeId<  210,  512,  128,  8,  0, CRM_0,  NMT_V,  0 >::ID,
+						// FIXME: Setting NMT_4 is incorrect but makes things work
+						NAMCOT_340                 = MakeId<  210,  256,  256,  0,  0, CRM_0,  NMT_4,  0 >::ID,
 						// Nitra
 						NITRA_TDA                  = MakeId<  250,  512,  256,  0,  0, CRM_0,  NMT_V,  0 >::ID,
 						// NTDEC
@@ -641,7 +644,7 @@ namespace Nes
 					Context(Cpu*,Apu*,Ppu*,Ram&,Ram&,const Ram&,Type::Nmt,bool,bool,Chips&);
 
 					bool DetectBoard(wcstring,dword);
-					bool DetectBoard(byte,dword,bool);
+					bool DetectBoard(byte,dword,bool,byte);
 
 					cstring name;
 					Type type;
