@@ -13,6 +13,11 @@ typedef struct {
 	int video_contrast;
 	int video_hue;
 	int video_ntsc_mode;
+	int video_ntsc_sharpness;
+	int video_ntsc_resolution;
+	int video_ntsc_bleed;
+	int video_ntsc_artifacts;
+	int video_ntsc_fringing;
 	int video_xbr_corner_rounding;
 	bool video_linear_filter;
 	bool video_tv_aspect;
@@ -53,6 +58,7 @@ typedef struct {
 	//bool misc_suppress_screensaver;
 	bool misc_genie_distortion;
 	bool misc_disable_gui;
+	bool misc_config_pause;
 } settings_t;
 
 void config_file_read();
