@@ -2,7 +2,14 @@
 #define _GTKUI_H_
 
 #include <gtk/gtk.h>
+
+#ifdef _MINGW
+#include <gdk/gdkwin32.h>
+#elif _APPLE
+#else
 #include <gdk/gdkx.h>
+#include <gdk/gdkwayland.h>
+#endif
 
 void gtkui_init(int argc, char *argv[]);
 void gtkui_create();
