@@ -1,7 +1,7 @@
 /*
  * Nestopia UE
  * 
- * Copyright (C) 2012-2014 R. Danbrook
+ * Copyright (C) 2012-2016 R. Danbrook
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,9 +36,12 @@
 extern settings_t conf;
 extern gamepad_t player[NUMGAMEPADS];
 extern char padpath[512];
+extern bool playing;
 extern bool confrunning;
 
 GtkWidget *configwindow;
+GtkWidget *notebook;
+gint tabnum = 0;
 
 // Audio
 GtkWidget *scale_audio_volume[NUMCHANNELS];
@@ -56,13 +59,14 @@ GtkWidget *gtkui_config() {
 	// Create the Configuration window
 	
 	if (configwindow) { return NULL; }
+	if (conf.misc_config_pause) { if (playing) { nst_pause(); } }
 	
 	configwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(configwindow), "Configuration");
 	
 	GtkWidget *box_upper = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	GtkWidget *box_lower = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-	GtkWidget *notebook = gtk_notebook_new();
+	notebook = gtk_notebook_new();
 	
 	gtk_container_add(GTK_CONTAINER(configwindow), box_upper);
 	
@@ -154,6 +158,7 @@ GtkWidget *gtkui_config() {
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_video_ntscmode), "Composite");
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_video_ntscmode), "S-Video");
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_video_ntscmode), "RGB");
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_video_ntscmode), "Custom");
 		
 	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_video_ntscmode), conf.video_ntsc_mode);
 	
@@ -1046,6 +1051,22 @@ GtkWidget *gtkui_config() {
 	g_signal_connect(G_OBJECT(check_misc_disable_gui), "toggled",
 		G_CALLBACK(gtkui_cb_misc_disable_gui), NULL);
 	
+	// Pause While Configuration Open
+	#ifndef _APPLE
+	GtkWidget *check_misc_config_pause = gtk_widget_new(
+				GTK_TYPE_CHECK_BUTTON,
+				"label", "Pause While Configuration Open",
+				"halign", GTK_ALIGN_START,
+				"margin-left", MARGIN_LR,
+				NULL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_misc_config_pause), conf.misc_config_pause);
+	
+	gtk_box_pack_start(GTK_BOX(box_misc), check_misc_config_pause, FALSE, FALSE, 0);
+	
+	g_signal_connect(G_OBJECT(check_misc_config_pause), "toggled",
+		G_CALLBACK(gtkui_cb_misc_config_pause), NULL);
+	#endif
+	
 	// Structuring the notebook
 	GtkWidget *label_video = gtk_label_new("Video");
 	GtkWidget *label_audio = gtk_label_new("Audio");
@@ -1081,13 +1102,17 @@ GtkWidget *gtkui_config() {
 	
 	gtk_widget_show_all(configwindow);
 	
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), tabnum);
+	
 	return configwindow;
 }
 
 void gtkui_config_ok() {
 	if (confrunning) { confrunning = false; }
+	tabnum = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
 	gtk_widget_destroy(configwindow);
 	configwindow = NULL;
+	nst_play();
 }
 
 void gtkui_audio_volume() {
