@@ -1,7 +1,7 @@
 /*
  * Nestopia UE
  * 
- * Copyright (C) 2012-2014 R. Danbrook
+ * Copyright (C) 2012-2016 R. Danbrook
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,10 +41,12 @@ extern nstpaths_t nstpaths;
 extern dimensions_t rendersize;
 extern SDL_DisplayMode displaymode;
 extern Emulator emulator;
+extern bool nst_nsf;
 
 bool confrunning, kbactivate = false;
 
 inputsettings_t inputconf;
+uiinput_t ui;
 gamepad_t player[NUMGAMEPADS];
 
 char inputconfpath[256];
@@ -79,8 +81,99 @@ static unsigned char nescodes[TOTALBUTTONS] = {
 
 void input_init() {
 	// Initialize input
-	Input(emulator).AutoSelectController(0);
-	Input(emulator).AutoSelectController(1);
+	
+	char controller[32];
+	
+	for (int i = 0; i < NUMGAMEPADS; i++) {
+		Input(emulator).AutoSelectController(i);
+		
+		switch(Input(emulator).GetConnectedController(i)) {
+			case Input::UNCONNECTED:
+				snprintf(controller, sizeof(controller), "%s", "Unconnected");
+				break;
+			case Input::PAD1:
+			case Input::PAD2:
+			case Input::PAD3:
+			case Input::PAD4:
+				snprintf(controller, sizeof(controller), "%s", "Standard Pad");
+				break;
+			case Input::ZAPPER:
+				snprintf(controller, sizeof(controller), "%s", "Zapper");
+				break;
+			case Input::PADDLE:
+				snprintf(controller, sizeof(controller), "%s", "Arkanoid Paddle");
+				break;
+			case Input::POWERPAD:
+				snprintf(controller, sizeof(controller), "%s", "Power Pad");
+				break;
+			case Input::POWERGLOVE:
+				snprintf(controller, sizeof(controller), "%s", "Power Glove");
+				break;
+			case Input::MOUSE:
+				snprintf(controller, sizeof(controller), "%s", "Mouse");
+				break;
+			case Input::ROB:
+				snprintf(controller, sizeof(controller), "%s", "R.O.B.");
+				break;
+			case Input::FAMILYTRAINER:
+				snprintf(controller, sizeof(controller), "%s", "Family Trainer");
+				break;
+			case Input::FAMILYKEYBOARD:
+				snprintf(controller, sizeof(controller), "%s", "Family Keyboard");
+				break;
+			case Input::SUBORKEYBOARD:
+				snprintf(controller, sizeof(controller), "%s", "Subor Keyboard");
+				break;
+			case Input::DOREMIKKOKEYBOARD:
+				snprintf(controller, sizeof(controller), "%s", "Doremikko Keyboard");
+				break;
+			case Input::HORITRACK:
+				snprintf(controller, sizeof(controller), "%s", "Hori Track");
+				break;
+			case Input::PACHINKO:
+				snprintf(controller, sizeof(controller), "%s", "Pachinko");
+				break;
+			case Input::OEKAKIDSTABLET:
+				snprintf(controller, sizeof(controller), "%s", "Oeka Kids Tablet");
+				break;
+			case Input::KONAMIHYPERSHOT:
+				snprintf(controller, sizeof(controller), "%s", "Konami Hypershot");
+				break;
+			case Input::BANDAIHYPERSHOT:
+				snprintf(controller, sizeof(controller), "%s", "Bandai Hypershot");
+				break;
+			case Input::CRAZYCLIMBER:
+				snprintf(controller, sizeof(controller), "%s", "Crazy Climber");
+				break;
+			case Input::MAHJONG:
+				snprintf(controller, sizeof(controller), "%s", "Mahjong");
+				break;
+			case Input::EXCITINGBOXING:
+				snprintf(controller, sizeof(controller), "%s", "Exciting Boxing");
+				break;
+			case Input::TOPRIDER:
+				snprintf(controller, sizeof(controller), "%s", "Top Rider");
+				break;
+			case Input::POKKUNMOGURAA:
+				snprintf(controller, sizeof(controller), "%s", "Pokkun Moguraa");
+				break;
+			case Input::PARTYTAP:
+				snprintf(controller, sizeof(controller), "%s", "PartyTap");
+				break;
+			case Input::TURBOFILE:
+				snprintf(controller, sizeof(controller), "%s", "Turbo File");
+				break;
+			case Input::BARCODEWORLD:
+				snprintf(controller, sizeof(controller), "%s", "Barcode World");
+				break;
+			default:
+				snprintf(controller, sizeof(controller), "%s", "Unknown");
+				break;
+		}
+		
+		fprintf(stderr, "Port %d: %s\n", i + 1, controller);
+	}
+	
 	video_set_cursor();
 }
 
@@ -392,41 +485,67 @@ void input_match_keyboard(Input::Controllers *controllers, SDL_Event event) {
 
 	input_inject(controllers, input);
 	
-	if (event.key.keysym.scancode == SDL_SCANCODE_GRAVE && event.type == SDL_KEYDOWN) { timing_set_altspeed(); }
-	if (event.key.keysym.scancode == SDL_SCANCODE_GRAVE && event.type == SDL_KEYUP) { timing_set_default(); }
+	if (event.key.keysym.scancode == ui.altspeed && event.type == SDL_KEYDOWN) { timing_set_altspeed(); }
+	if (event.key.keysym.scancode == ui.altspeed && event.type == SDL_KEYUP) { timing_set_default(); }
 	
 	const Uint8 *keys = SDL_GetKeyboardState(NULL);
 	
 	// Insert Coins
 	controllers->vsSystem.insertCoin = 0;
-	if (keys[SDL_SCANCODE_F1]) { controllers->vsSystem.insertCoin |= Input::Controllers::VsSystem::COIN_1; }
-	if (keys[SDL_SCANCODE_F2]) { controllers->vsSystem.insertCoin |= Input::Controllers::VsSystem::COIN_2; }
+	if (keys[ui.insertcoin1]) { controllers->vsSystem.insertCoin |= Input::Controllers::VsSystem::COIN_1; }
+	if (keys[ui.insertcoin2]) { controllers->vsSystem.insertCoin |= Input::Controllers::VsSystem::COIN_2; }
 	
 	// Process non-game events
-	if (keys[SDL_SCANCODE_F3]) { nst_flip_disk(); }
-	if (keys[SDL_SCANCODE_F4]) { nst_switch_disk(); }
-	if (keys[SDL_SCANCODE_F5]) { nst_state_quicksave(0); }
-	if (keys[SDL_SCANCODE_F6]) { nst_state_quicksave(1); }
-	if (keys[SDL_SCANCODE_F7]) { nst_state_quickload(0); }
-	if (keys[SDL_SCANCODE_F8]) { nst_state_quickload(1); }
-	//if (keys[SDL_SCANCODE_F9]) {  }
-	//if (keys[SDL_SCANCODE_F10]) {  }
-	//if (keys[SDL_SCANCODE_F11]) {  }
-	if (keys[SDL_SCANCODE_F12]) { nst_reset(0); }
+	if (keys[ui.fdsflip]) { nst_flip_disk(); }
+	if (keys[ui.fdsswitch]) { nst_switch_disk(); }
+	if (keys[ui.qsave1]) { nst_state_quicksave(0); }
+	if (keys[ui.qsave2]) { nst_state_quicksave(1); }
+	if (keys[ui.qload1]) { nst_state_quickload(0); }
+	if (keys[ui.qload2]) { nst_state_quickload(1); }
+	
+	// Screenshot
+	if (keys[ui.screenshot]) { video_screenshot(NULL); }
+	
+	// Reset
+	if (keys[ui.reset]) { nst_reset(0); }
 	
 	// Rewinder
-	if (keys[SDL_SCANCODE_BACKSPACE]) { nst_set_rewind(0); }
-	if (keys[SDL_SCANCODE_BACKSLASH]) { nst_set_rewind(1); }
+	if (keys[ui.rwstart]) { nst_set_rewind(0); }
+	if (keys[ui.rwstop]) { nst_set_rewind(1); }
+	
+	// Video
+	if (keys[ui.fullscreen]) { video_toggle_fullscreen(); }
+	if (keys[ui.filter]) { video_toggle_filter(); }
+	if (keys[ui.scalefactor]) { video_toggle_scalefactor(); }
+	
+	// NSF
+	if (nst_nsf) {
+		Nsf nsf(emulator);
+		
+		if (keys[SDL_SCANCODE_UP]) {
+			nsf.PlaySong();
+			video_clear_buffer();
+			video_disp_nsf();
+		}
+		if (keys[SDL_SCANCODE_DOWN]) {
+			//nsf.StopSong();
+		}
+		if (keys[SDL_SCANCODE_LEFT]) {
+			nsf.SelectPrevSong();
+			video_clear_buffer();
+			video_disp_nsf();
+		}
+		if (keys[SDL_SCANCODE_RIGHT]) {
+			nsf.SelectNextSong();
+			video_clear_buffer();
+			video_disp_nsf();
+		}
+	}
 	
 	// Escape exits when not in GUI mode
 	if (keys[SDL_SCANCODE_ESCAPE]) {
 		if (conf.misc_disable_gui) { nst_schedule_quit(); }
 	}
-	
-	// F toggles fullscreen
-	if (keys[SDL_SCANCODE_F]) { video_toggle_fullscreen(); }
-	if (keys[SDL_SCANCODE_T]) { video_toggle_filter(); }
-	if (keys[SDL_SCANCODE_G]) { video_toggle_scalefactor(); }
 }
 
 void input_match_mouse(Input::Controllers *controllers, SDL_Event event) {
@@ -511,7 +630,7 @@ char* input_translate_event(SDL_Event event) {
 		
 		case SDL_JOYBUTTONUP:	
 		case SDL_JOYBUTTONDOWN:
-			sprintf(inputcode, "j%db%x", event.jbutton.which, event.jbutton.button);
+			sprintf(inputcode, "j%db%d", event.jbutton.which, event.jbutton.button);
 			break;
 	}
 	return inputcode;
@@ -535,6 +654,9 @@ SDL_Event input_translate_string(char *string) {
 	else if ((unsigned char)string[2] == 0x62) { // Button
 		which = string[1] - '0';
 		value = string[3] - '0';
+		if (string[4]) {
+			value = ((string[3] - '0') * 10) + (string[4] - '0');
+		}
 		event.type = SDL_JOYBUTTONDOWN;
 		event.jbutton.which = which;
 		event.jbutton.button = value;
@@ -572,6 +694,30 @@ void input_config_read() {
 	else {
 		// Map the input settings from the config file
 		
+		// User Interface
+		ui.qsave1 = SDL_GetScancodeFromName(inputconf.qsave1);
+		ui.qsave2 = SDL_GetScancodeFromName(inputconf.qsave2);
+		ui.qload1 = SDL_GetScancodeFromName(inputconf.qload1);
+		ui.qload2 = SDL_GetScancodeFromName(inputconf.qload2);
+		
+		ui.screenshot = SDL_GetScancodeFromName(inputconf.screenshot);
+		
+		ui.fdsflip = SDL_GetScancodeFromName(inputconf.fdsflip);
+		ui.fdsswitch = SDL_GetScancodeFromName(inputconf.fdsswitch);
+		
+		ui.insertcoin1 = SDL_GetScancodeFromName(inputconf.insertcoin1);
+		ui.insertcoin2 = SDL_GetScancodeFromName(inputconf.insertcoin2);
+		
+		ui.reset = SDL_GetScancodeFromName(inputconf.reset);
+		
+		ui.altspeed = SDL_GetScancodeFromName(inputconf.altspeed);
+		ui.rwstart = SDL_GetScancodeFromName(inputconf.rwstart);
+		ui.rwstop = SDL_GetScancodeFromName(inputconf.rwstop);
+		
+		ui.fullscreen = SDL_GetScancodeFromName(inputconf.fullscreen);
+		ui.filter = SDL_GetScancodeFromName(inputconf.filter);
+		ui.scalefactor = SDL_GetScancodeFromName(inputconf.scalefactor);
+		
 		// Player 1
 		player[0].u = SDL_GetScancodeFromName(inputconf.kb_p1u);
 		player[0].d = SDL_GetScancodeFromName(inputconf.kb_p1d);
@@ -629,6 +775,32 @@ void input_config_write() {
 		fprintf(fp, "; Possible values for keyboard input are in the Key Name column:\n; https://wiki.libsdl.org/SDL_Scancode\n\n");
 		fprintf(fp, "; Possible values for joystick input:\n; j[joystick number][a|b|h][button/hat/axis number][1/0 = +/- (axes only)]\n");
 		fprintf(fp, "; Example: j0b3 = joystick 0, button 3. j1a11 = joystick 1, axis 1 +\n\n");
+		
+		fprintf(fp, "[ui]\n");
+		fprintf(fp, "qsave1=%s\n", SDL_GetScancodeName(ui.qsave1));
+		fprintf(fp, "qsave2=%s\n", SDL_GetScancodeName(ui.qsave2));
+		fprintf(fp, "qload1=%s\n", SDL_GetScancodeName(ui.qload1));
+		fprintf(fp, "qload2=%s\n", SDL_GetScancodeName(ui.qload2));
+		
+		fprintf(fp, "screenshot=%s\n", SDL_GetScancodeName(ui.screenshot));
+		
+		fprintf(fp, "fdsflip=%s\n", SDL_GetScancodeName(ui.fdsflip));
+		fprintf(fp, "fdsswitch=%s\n", SDL_GetScancodeName(ui.fdsswitch));
+		
+		fprintf(fp, "insertcoin1=%s\n", SDL_GetScancodeName(ui.insertcoin1));
+		fprintf(fp, "insertcoin2=%s\n", SDL_GetScancodeName(ui.insertcoin2));
+		
+		fprintf(fp, "reset=%s\n", SDL_GetScancodeName(ui.reset));
+		
+		fprintf(fp, "altspeed=%s\n", SDL_GetScancodeName(ui.altspeed));
+		fprintf(fp, "rwstart=%s\n", SDL_GetScancodeName(ui.rwstart));
+		fprintf(fp, "rwstop=%s\n", SDL_GetScancodeName(ui.rwstop));
+		
+		fprintf(fp, "fullscreen=%s\n", SDL_GetScancodeName(ui.fullscreen));
+		fprintf(fp, "filter=%s\n", SDL_GetScancodeName(ui.filter));
+		fprintf(fp, "scalefactor=%s\n", SDL_GetScancodeName(ui.scalefactor));
+		fprintf(fp, "\n"); // End of Section
+		
 		fprintf(fp, "[gamepad1]\n");
 		fprintf(fp, "kb_u=%s\n", SDL_GetScancodeName(player[0].u));
 		fprintf(fp, "kb_d=%s\n", SDL_GetScancodeName(player[0].d));
@@ -682,13 +854,37 @@ void input_config_write() {
 }
 
 void input_set_default() {
-	// Set default input config	
+	// Set default input config
+	
+	ui.qsave1 = SDL_GetScancodeFromName("F5");
+	ui.qsave2 = SDL_GetScancodeFromName("F6");
+	ui.qload1 = SDL_GetScancodeFromName("F7");
+	ui.qload2 = SDL_GetScancodeFromName("F8");
+	
+	ui.screenshot = SDL_GetScancodeFromName("F9");
+	
+	ui.fdsflip = SDL_GetScancodeFromName("F3");
+	ui.fdsswitch = SDL_GetScancodeFromName("F4");
+	
+	ui.insertcoin1 = SDL_GetScancodeFromName("F1");
+	ui.insertcoin2 = SDL_GetScancodeFromName("F2");
+	
+	ui.reset = SDL_GetScancodeFromName("F12");
+	
+	ui.altspeed = SDL_GetScancodeFromName("`");
+	ui.rwstart = SDL_GetScancodeFromName("Backspace");
+	ui.rwstop = SDL_GetScancodeFromName("\\");
+	
+	ui.fullscreen = SDL_GetScancodeFromName("F");
+	ui.filter = SDL_GetScancodeFromName("T");
+	ui.scalefactor = SDL_GetScancodeFromName("G");
+	
 	player[0].u = SDL_GetScancodeFromName("Up");
 	player[0].d = SDL_GetScancodeFromName("Down");
 	player[0].l = SDL_GetScancodeFromName("Left");
 	player[0].r = SDL_GetScancodeFromName("Right");
 	player[0].select = SDL_GetScancodeFromName("Right Shift");
-	player[0].start = SDL_GetScancodeFromName("Right Ctrl");
+	player[0].start = SDL_GetScancodeFromName("Return");
 	player[0].a = SDL_GetScancodeFromName("Z");
 	player[0].b = SDL_GetScancodeFromName("A");
 	player[0].ta = SDL_GetScancodeFromName("X");
@@ -732,8 +928,32 @@ static int input_config_match(void* user, const char* section, const char* name,
 	// Match values from input config file and populate live config
 	inputsettings_t* pconfig = (inputsettings_t*)user;
 	
+	// User Interface
+	if (MATCH("ui", "qsave1")) { pconfig->qsave1 = strdup(value); }
+	else if (MATCH("ui", "qsave2")) { pconfig->qsave2 = strdup(value); }
+	else if (MATCH("ui", "qload1")) { pconfig->qload1 = strdup(value); }
+	else if (MATCH("ui", "qload2")) { pconfig->qload2 = strdup(value); }
+	
+	else if (MATCH("ui", "screenshot")) { pconfig->screenshot = strdup(value); }
+	
+	else if (MATCH("ui", "fdsflip")) { pconfig->fdsflip = strdup(value); }
+	else if (MATCH("ui", "fdsswitch")) { pconfig->fdsswitch = strdup(value); }
+	
+	else if (MATCH("ui", "insertcoin1")) { pconfig->insertcoin1 = strdup(value); }
+	else if (MATCH("ui", "insertcoin2")) { pconfig->insertcoin2 = strdup(value); }
+	
+	else if (MATCH("ui", "reset")) { pconfig->reset = strdup(value); }
+	
+	else if (MATCH("ui", "altspeed")) { pconfig->altspeed = strdup(value); }
+	else if (MATCH("ui", "rwstart")) { pconfig->rwstart = strdup(value); }
+	else if (MATCH("ui", "rwstop")) { pconfig->rwstop = strdup(value); }
+	
+	else if (MATCH("ui", "fullscreen")) { pconfig->fullscreen = strdup(value); }
+	else if (MATCH("ui", "filter")) { pconfig->filter = strdup(value); }
+	else if (MATCH("ui", "scalefactor")) { pconfig->scalefactor = strdup(value); }
+	
 	// Player 1
-	if (MATCH("gamepad1", "kb_u")) { pconfig->kb_p1u = strdup(value); }
+	else if (MATCH("gamepad1", "kb_u")) { pconfig->kb_p1u = strdup(value); }
 	else if (MATCH("gamepad1", "kb_d")) { pconfig->kb_p1d = strdup(value); }
 	else if (MATCH("gamepad1", "kb_l")) { pconfig->kb_p1l = strdup(value); }
 	else if (MATCH("gamepad1", "kb_r")) { pconfig->kb_p1r = strdup(value); }
