@@ -1,7 +1,7 @@
 #ifndef _MAIN_H_
 #define _MAIN_H_
 
-#define VERSION "1.46.2"
+#define VERSION "1.47"
 
 typedef struct {
 	char nstdir[256];
@@ -44,6 +44,4 @@ void nst_switch_disk();
 
 void nst_dipswitch();
 
-void SetupInput();
-
 #endif
