@@ -7,10 +7,13 @@
 #define OVERSCAN_BOTTOM 8
 #define OVERSCAN_TOP 8
 
+#include <epoxy/gl.h>
 #include <SDL.h>
 #include <SDL_opengl.h>
 #include <SDL_endian.h>
-#include "GL/glu.h"
+#ifdef _APPLE
+#include <OpenGL/gl.h>
+#endif
 
 // This is part of an elaborate hack to embed the SDL window
 struct SDL_Window {
@@ -32,14 +35,15 @@ typedef struct {
 	int h;
 } dimensions_t;
 
-void opengl_init_structures();
-void opengl_cleanup();
-void opengl_blit();
+void ogl_init();
+void ogl_deinit();
+void ogl_render();
 
 void video_init();
 void video_create_standalone();
 void video_create_embedded();
 void video_create();
+void video_swapbuffers();
 void video_destroy();
 void video_toggle_fullscreen();
 void video_toggle_filter();
@@ -52,5 +56,10 @@ void video_set_title(const char *title);
 
 long video_lock_screen(void*& ptr);
 void video_unlock_screen(void*);
+void video_screenshot(const char* filename);
+void video_clear_buffer();
+void video_disp_nsf();
+void video_text_draw(const char *text, int xpos, int ypos);
+void video_text_match(const char *text, int *xpos, int *ypos, int strpos);
 
 #endif
